/*********************************************************************
 *
 *                             NOTICE
 *               COPYRIGHT 2021 Revulytics, Inc
 *                UNPUBLISHED - ALL RIGHTS RESERVED
 * 
 *      This program is confidential, proprietary, and a trade
 *   secret of Revulytics, Inc. The receipt or possession of
 *     this program does not convey any rights to reproduce or
 *      disclose its contents, or to manufacture, use, or sell
 *       anything that it may describe, in whole or in part, 
 *   without the specific written consent of Revulytics, Inc.
 ********************************************************************/
/**
 * @brief    ADAPTER PATTERN:  Class for Objective-C interface for the RUI SDK.
 * The RUI SDK supports various interfaces, where each interface is aligned to the types and conventions of a particular programming
 * language.  A client application uses the RUI SDK by selecting one of the interfaces.  This file contains the API for the Objective-C interface,
 * which can be used by an Objective-C compiler (OSX).  Each interface is supported by the common definitions in RUISDKDefines.h.
 */
#import "ruiSDKDefines.h"
#import <Foundation/Foundation.h>

#define RUI_COMBINER(x, y)              x ## y
#define RUI_XCOMBINER(x, y)             RUI_COMBINER(x, y)
#define RUI_REACHOUT_HANDLER_FULLNAME   RUI_XCOMBINER(RUIReachOutHandlerOBJC_, RUI_SDK_SUFFIX)
#define RUIReachOutHandlerOBJC          RUI_REACHOUT_HANDLER_FULLNAME
#define RUI_EXCEPTION_EVENT_FULLNAME    RUI_XCOMBINER(RUIExceptionEventOBJC_, RUI_SDK_SUFFIX)
#define RUIExceptionEventOBJC           RUI_EXCEPTION_EVENT_FULLNAME
#define RUI_NAME_VALUE_PAIR_FULLNAME    RUI_XCOMBINER(RUINameValuePairOBJC_, RUI_SDK_SUFFIX)
#define RUINameValuePairOBJC            RUI_NAME_VALUE_PAIR_FULLNAME
#define RUISDKOBJC                      RUI_XCOMBINER(RUISDKOBJC_, RUI_SDK_SUFFIX)

#define RUIRESULTOBJC      int32_t
#define RUISTATEOBJC       int32_t

/**
 * INTERFACE PATTERN:  Interface for a ReachOut Handler, that must be implemented to create a custom handler.  No implementation
 * is required for the default graphical ReachOut handler.
 * @see RUISDKOBJC::initRegisterDefaultGraphicalReachOutHandler:()
 * @see RUISDKOBJC::setReachOutHandler:()
 */
@protocol RUI_REACHOUT_HANDLER_FULLNAME <NSObject>
/**
 * Callback function used to handle a ReachOut™ from the RUI Server when registering a custom ReachOut™ Handler
 * (RUISDKOBJC::setReachOutHandler:()).  This callback is called once for each ReachOut™ received.
 *
 * The callback can be implemented synchronously or asynchronously, depending on the handler functionality.  Another ReachOut™
 * will not be delivered to the handler until the handler has returned -AND- readyForNext is set to a non-zero value.  readyForNext
 * is initialized to zero value before the call to handleWidth:height:position:message:readyForNext:().  Therefore, for a
 * synchronous handler, the value needs to be changed before the handler returns.  For an asynchronous handler, readyForNext can
 * be left alone before the handler returns, and then set to non-zero asynchronously when the handler has completed processing of
 * the ReachOut™ and ready for another ReachOut™.
 *
 * @see RUISDKOBJC::setReachOutHandler:()
 * @param[in]  width         The ReachOut™ data: either a number of pixels or percentage; see ReachOut™ Data for more details.
 * @param[in]  height        The ReachOut™ data: either a number of pixels or percentage; see ReachOut™ Data for more details.
 * @param[in]  position      The ReachOut™ data: 1-9; see ReachOut™ Data for more details.
 * @param[in]  message       The ReachOut™ data: URL or text; see ReachOut™ Data for more details.
 * @throws     None.
 * @returns    Nothing.
 */
-(void)handleWidth:(NSString*)width height:(NSString*)height position:(int32_t)position message:(NSString*)message;

/**
 * Callback function used to handle SDK shutdown when registering a custom ReachOut™ Handler (RUISDKOBJC::setReachOutHandler:()).
 * This callback function will be called exactly once, no matter if callback handleWidth:height:position:message:readyForNext:()
 * was ever called or not.  After close() is called, handleWidth:height:position:message:readyForNext:() will not be called again.
 *
 * @see RUISDKOBJC::setReachOutHandler:()
 * @throws     None.
 * @returns    Nothing.
 */
-(void)close;    

/**
 * Callback function used to indicate if the ReachOut™ Handler (RUISDKOBJC::setReachOutHandler:()) is ready for a new message.
 *
 * @see RUISDKOBJC::setReachOutHandler:()
 * @throws     None.
 * @returns    Integer with 0 value indicates ReachOut™ Handler is not ready for a new message. Non zero indicate it is ready.
 */
-(int32_t)getReadyForNext;    
@end

/**
 * Convenience type for the data associated with event tracking for an exception.
 * @see RUISDKOBJC::trackException:sessionID:()
 */
@interface RUI_EXCEPTION_EVENT_FULLNAME : NSObject <NSCopying>
@property(nonatomic, copy) NSString* className;
@property(nonatomic, copy) NSString* methodName;
@property(nonatomic, copy) NSString* exceptionMessage;
@property(nonatomic, copy) NSString* stackTrace;
-(id)init;
-(id)initWithClassName:(NSString*)className methodName:(NSString*)methodName exceptionMessage:(NSString*)exceptionMessage stackTrace:(NSString*)stackTrace;
-(id)copyWithZone:(NSZone*)zone;
-(void)dealloc;
@end

/**
 * Convenience type for the data associated with event tracking with custom data.
 * @see RUISDKOBJC::trackEventCategory:eventName:withNameValuePairs:sesionID:()
 */
@interface RUI_NAME_VALUE_PAIR_FULLNAME : NSObject <NSCopying>
@property(nonatomic, copy) NSString* name;
@property(nonatomic, copy) NSString* value;
-(id)init;
-(id)initWithName:(NSString*)name value:(NSString*)value;
-(id)copyWithZone:(NSZone*)zone;
-(void)dealloc;
@end

/**
 * ADAPTER PATTERN:  Class to be instantiated and used for an instance of the RUI SDK.
 */
@interface RUISDKOBJC : NSObject
{
    //Intentionally empty.
}

/**
 * Constructor:  Creates an instance of the SDK.  Constructor must be paired with call to destructor when the client application
 * is done using the RUI SDK.  The constructor does not configure the RUI SDK
 * (createConfig:productID:appName:serverURL:protocol:aesKeyHex:multiSessionEnabled:reachOutOnAutoSync:()) nor start the RUI SDK
 * (startSDK()).
 *
 * A typical client will create only a single instance of the RUI SDK.  Creating more than one RUI SDK instance is allowed and
 * is used to support clients that are plug-ins or other scenarios whereby multiple independent clients may co-exist in the same
 * executable.  Multiple RUI SDK instances perform independently of one another with the potential exception of shared or
 * unshared configuration file (createConfig:productID:appName:serverURL:protocol:aesKeyHex:multiSessionEnabled:reachOutOnAutoSync:()).
 *
 * The constructor is a synchronous function, returning when all functionality is completed.
 *
 * @see createConfig:productID:appName:serverURL:protocol:aesKeyHex:multiSessionEnabled:reachOutOnAutoSync:()
 * @see startSDK()
 * @param[in] registerHandler On platforms that contain a RUI SDK default graphical ReachOut™ handler,
 *                            automatically register that handler via setReachOutHandler:().
 * @returns id for instantiated object, or nil if there is a problem constructing the SDK instance.
 */
-(id)initRegisterDefaultGraphicalReachOutHandler:(BOOL)registerHandler;

/**
 * Destructor:  Destroys an instance of the SDK, created with the constructor.
 *
 * The destructor is a synchronous function, returning when all functionality is completed.
 */
-(void)dealloc;

/**
 * Returns the version information for the RUI SDK instance in the supplied string parameter.
 *
 * createSDKVersionString() can be called more than once.
 *
 * createSDKVersionString() is a synchronous function, returning when all functionality is completed.
 *
 * @returns    The version string for the RUI SDK; caller takes ownership.
 */
-(NSString*)createSDKVersionString;

/**
 * Returns the current state of the RUI SDK instance.  The RUI SDK state can change asynchronously.
 *
 * getState() can be called more than once.
 *
 * getState() is a synchronous function, returning when all functionality is completed.
 *
 * @returns    One of the state codes in RUISDKDefines.h:
 *   * RUI_SDK_STATE_FATAL_ERROR                             Irrecoverable internal fatal error.  No futher API calls should be made.
 *   * RUI_SDK_STATE_UNINITIALIZED                           Instance successfully created (initRegisterDefaultGraphicalReachOutHandler:())
 *               but not yet successfully configured (createConfig:productID:appName:serverURL:protocol:aesKeyHex:multiSessionEnabled:reachOutOnAutoSync:()).
 *   * RUI_SDK_STATE_CONFIG_INITIALIZED_NOT_STARTED          Successfully configured (createConfig:productID:appName:serverURL:protocol:aesKeyHex:multiSessionEnabled:reachOutOnAutoSync:())
 *               and not yet started (startSDK()).  Will be normal start.
 *   * RUI_SDK_STATE_CONFIG_MISSING_OR_CORRUPT_NOT_STARTED   Successfully configured (createConfig:productID:appName:serverURL:protocol:aesKeyHex:multiSessionEnabled:reachOutOnAutoSync:())
 *               and not yet started (startSDK()).  Will be a New Registration start.
 *   * RUI_SDK_STATE_STARTED_NEW_REG_RUNNING                 Running (startSDK()) with New Registration in progress, not yet completed.
 *   * RUI_SDK_STATE_RUNNING                                 Running (startSDK()) with no need for New Registration or with successfully completd New Registration.
 *   * RUI_SDK_STATE_ABORTED_NEW_REG_PROXY_AUTH_FAILURE      Aborted run (startSDK()) due to failed New Registration
 *               (createConfig:productID:appName:serverURL:protocol:aesKeyHex:multiSessionEnabled:reachOutOnAutoSync:()).
 *   * RUI_SDK_STATE_ABORTED_NEW_REG_NETWORK_FAILURE         Aborted run (startSDK()) due to failed New Registration
 *               (createConfig:productID:appName:serverURL:protocol:aesKeyHex:multiSessionEnabled:reachOutOnAutoSync:()).
 *   * RUI_SDK_STATE_ABORTED_NEW_REG_FAILED                  Aborted run (startSDK()) due to failed New Registration
 *               (createConfig:productID:appName:serverURL:protocol:aesKeyHex:multiSessionEnabled:reachOutOnAutoSync:()).
 *   * RUI_SDK_STATE_SUSPENDED                               Instance has been instructed by RUI Server to backoff.  Will return to Running once backoff cleared.
 *   * RUI_SDK_STATE_PERMANENTLY_DISABLED                    Instance has been instructed by RUI Server to disable.  This is permanent, irrecoverable state.
 *   * RUI_SDK_STATE_STOPPING_NON_SYNC                       Stop in progress (stopSDK:()).  Stopping non-Sync-related threads.
 *   * RUI_SDK_STATE_STOPPING_ALL                            Stop in progress (stopSDK:()).  Stopping Sync-related threads.
 *   * RUI_SDK_STATE_STOPPED                                 Stop completed (stopSDK:()).
 *   * RUI_SDK_STATE_OPTED_OUT                               Instance has been instructed by client to opt out.
 */
-(RUISTATEOBJC)getState;

/**
 * Returns the client ID for the RUI SDK instance.
 *
 * getClientID() can be called more than once.
 *
 * getClientID() is a synchronous function, returning when all functionality is completed.
 *
 * @returns    The client ID for the RUI SDK; caller takes ownership.
 */
-(NSString*)getClientID;

/**
 * Sets a custom ReachOut™ handler.  Any previously registered handler, including the default graphical ReachOut™ handler that
 * may have been registered (initRegisterDefaultGraphicalReachOutHandler:()).  Setting a handler to nil effectively removes the
 * current handler, if any, without setting a new handler.
 *
 * setReachOutHandler:() can be called more than once.
 *
 * setReachOutHandler:() is a synchronous function, returning when all functionality is completed.
 *
 * @see RUI_REACHOUT_HANDLER_FULLNAME
 * @param[in]  handler     An instance implementing the RUIReachOutHandlerOBJC protocol.
 * @returns    One of the result codes in RUISDKDefines.h:
 *   * RUI_OK                                        Function (which may be synchronous or asynchronous), fully successful during synchronous functionality.
 *   * RUI_SDK_INTERNAL_ERROR_FATAL                  Error:  Irrecoverable internal fatal error.  No further API calls should be made.
 *   * RUI_SDK_SUSPENDED                             Function validation: The RUI Server has instructed a temporary backoff.
 *   * RUI_SDK_PERMANENTLY_DISABLED                  Function validation: The RUI Server has instructed a permanent disable.
 *   * RUI_SDK_OPTED_OUT                             Function validation: The application has instructed an opt out.
 *   * RUI_SDK_ALREADY_STARTED                       Function validation: SDK has already been successfully started.
 *   * RUI_SDK_ALREADY_STOPPED                       Function validation: SDK has already been successfully stopped.
 */
-(RUIRESULTOBJC)setReachOutHandler:(id<RUI_REACHOUT_HANDLER_FULLNAME>)handler;

/**
 * Creates a configuration for the RUI SDK instance.  A configuration is passed on a file, specified by configFilePath,
 * productID, and appName.  If two or more RUI SDK instances, in the same process or in different processes, use the same values
 * for these three parameters, then those RUI SDK instances are bound together through a shared configuration.  That is not
 * generally desirable nor recommended.  There is no internal locking across multiple RUI SDK instances to manage that would-be shared
 * configuration file.  Client applications are responsible for synchronizing any file changes that are required.  Please
 * contact Revulytics for more information.
 *
 * The RUI SDK has two communications modes:  HTTPS or HTTP + AES-128 encryption.  The mode is configured by protocol.  When
 * protocol is RUI_PROTOCOL_HTTP_PLUS_ENCRYPTION or RUI_PROTOCOL_HTTPS_WITH_FALLBACK, the AES key must be supplied as a 128-bit
 * hex-encoded string (aesKeyHex).  When protocol is RUI_PROTOCOL_HTTPS, then aesKeyHex must be empty.
 *
 * On first execution of a client application, no RUI SDK configuration file will exist.  This situation is detected by the RUI
 * SDK and will result in a New Registration message to the RUI Server at startSDK().  Once the configuration is received
 * from the RUI Server, the RUI SDK writes the configuration file, that is then used for subsequent client application
 * executions.
 *
 * createConfig:productID:appName:serverURL:protocol:aesKeyHex:multiSessionEnabled:reachOutOnAutoSync:() must be called before
 * most other APIs and must only be successfully called once.
 *
 * createConfig:productID:appName:serverURL:protocol:aesKeyHex:multiSessionEnabled:reachOutOnAutoSync:() is a synchronous
 * function, returning when all functionality is completed.
 *
 * @see setReachOutHandler:()
 * @param[in] configFilePath      The directory to use for the RUI SDK instance's configuration file.  Cannot be empty; must
 *                                exist and be writeable.
 * @param[in] productID           The Revulytics-supplied product id for this client; 10 digits.
 * @param[in] appName             The customer-supplied application name for this client, to distinguish suites with the same
 *                                productID.  Cannot be empty or contain white space; at most 16 UTF8 characters.
 * @param[in] serverURL           The URL to use for New Registrations.  Subsequent communications from the RUI SDK will either
 *                                use this URL or the URL supplied by the RUI Server).  Cannot be empty.  Cannot have a
 *                                protocol prefix (e.g., no "http://" or "https://").
 * @param[in] protocol            Indicates whether HTTP + AES, HTTPS with fallback to HTTP + AES, or HTTPS is used to
 *                                communicate with the RUI Server.
 * @param[in] aesKeyHex           AES Key to use when protocol includes encryption; 32 hex chars (16 bytes, 128 bit) key.
 * @param[in] multiSessionEnabled Indicates whether or not the client will explicitly manage sessionID's via startSession:()
 *                                and stopSession:(), and supply those sessionID's to the various event tracking APIs.
 * @param[in] reachOutOnAutoSync  Indicates whether or not a ReachOut™ should be requested as part of each RUI SDK Automatic Sync.
 *                                A ReachOut™ request will be made only if a ReachOut™ handler has been set by registering the
 *                                default graphical handler (initRegisterDefaultGraphicalReachOutHandler:()) or a custom handler
                                  (setReachOutHandler:()).
 * @returns   One of the result codes in RUISDKDefines.h:
 *   * RUI_OK                                        Function (which may be synchronous or asynchronous), fully successful during synchronous functionality.
 *   * RUI_SDK_INTERNAL_ERROR_FATAL                  Error:  Irrecoverable internal fatal error.  No further API calls should be made.
 *   * RUI_SDK_PERMANENTLY_DISABLED                  Function validation: The RUI Server has instructed a permanent disable.
 *   * RUI_CONFIG_ALREADY_CREATED                    Function validation: Configuration has already been successfully created.
 *   * RUI_INVALID_PARAMETER_EXPECTED_NON_EMPTY      Parameter validation: Some API parameter is expected to be non-empty, and is not.
 *   * RUI_INVALID_PARAMETER_EXPECTED_NO_WHITESPACE  Parameter validation: Some API parameter is expected to be free of whitespace, and is not.
 *   * RUI_INVALID_PARAMETER_TOO_LONG                Parameter validation: Some API parameter violates its allowable max length.
 *   * RUI_INVALID_CONFIG_PATH                       Parameter validation: The configFilePath is not a well-formed directory name.
 *   * RUI_INVALID_CONFIG_PATH_NONEXISTENT_DIR       Parameter validation: The configFilePath identifies a directory that does not exist.
 *   * RUI_INVALID_CONFIG_PATH_NOT_WRITABLE          Parameter validation: The configFilePath identifies a directory that is not writable.
 *   * RUI_INVALID_PRODUCT_ID                        Parameter validation: The productID is not a well-formed Revulytics Product ID.
 *   * RUI_INVALID_SERVER_URL                        Parameter validation: The serverURL is not a well-formed URL.
 *   * RUI_INVALID_PROTOCOL                          Parameter validation: The protocol is not a legal value.
 *   * RUI_INVALID_AES_KEY_EXPECTED_EMPTY            Parameter validation: The AES Key is expected to be NULL/empty, and it not.
 *   * RUI_INVALID_AES_KEY_LENGTH                    Parameter validation: The AES Key is not the expected length (32 hex chars; 16 bytes).
 *   * RUI_INVALID_AES_KEY_FORMAT                    Parameter validation: The AES Key is not valid hex encoding.
 */
-(RUIRESULTOBJC)createConfig:(NSString*)configFilePath
                   productID:(NSString*)productID
                     appName:(NSString*)appName
                   serverURL:(NSString*)serverURL
                    protocol:(int32_t)protocol
                   aesKeyHex:(NSString*)aesKeyHex
         multiSessionEnabled:(BOOL)multiSessionEnabled
          reachOutOnAutoSync:(BOOL)reachOutOnAutoSync;

/**
 * Sets or clears the data to be used with a proxy.  If there is no proxy between the RUI SDK and the RUI Server,
 * there is no need to use this function.  address can be either empty (unnamed proxy) or non-empty (named proxy).
 * username and password must both be empty (non-authenticating proxy) or both be non-empty (authenticating proxy).
 * The port is only used for a named proxy, hence port must be zero if address is empty, otherwise port must be non-zero.
 * The RUI SDK uses the proxy data in multiple ways to attempt to communicate via a proxy.  The allowed parameter combinations
 * and their usage are the following:
 *   * address,        port,  username,       password   
 *   * empty,          0,     empty,          empty          - Resets the proxy data to its initial state, no proxy server is
 *                                                             used, and the RUI Server is contacted directly.
 *   * non-empty,      not 0, empty,          empty          - Identifies a non-authenticating named proxy that will be used
 *                                                             unless communications fails, then falling back to using no proxy.
 *   * empty,          0,     non-empty,      non-empty      - Identifies an authenticating unnamed proxy that will be used
 *                                                             unless communications fails, then falling back to using no proxy.
 *   * non-empty,      not 0, non-empty,      non-empty      - Identifies an authenticating named proxy that will be used
 *                                                             unless communications fails, then falling back to using an
 *                                                             authenticating unnamed proxy, then falling back to using no proxy.
 *
 * setProxyAddress:port:username:password:() can be called between
 * createConfig:productID:appName:serverURL:protocol:aesKeyHex:multiSessionEnabled:reachOutOnAutoSync:() and stopSDK(), and can
 * be called zero or more times.
 *
 * setProxyAddress:port:username:password:() is a synchronous function, returning when all functionality is completed.
 *
 * @param[in] address      The server name or IP address (dot notation) for the named proxy.
 * @param[in] port         The port for the proxy server; only used with named proxy, port != 0 if and only if address non-empty.
 * @param[in] username     The named proxy username; username and password must both be empty or both be non-empty.
 * @param[in] password     The named proxy password; username and password must both be empty or both be non-empty.
 * @returns   One of the result codes in RUISDKDefines.h:
 *   * RUI_OK                                        Function (which may be synchronous or asynchronous), fully successful during synchronous functionality.
 *   * RUI_SDK_INTERNAL_ERROR_FATAL                  Error:  Irrecoverable internal fatal error.  No further API calls should be made.
 *   * RUI_SDK_ABORTED                               Error:  A required New Registration has failed, and hence the SDK is aborted.  StopSDK and DestroySDK are possible.
 *   * RUI_SDK_PERMANENTLY_DISABLED                  Function validation: The RUI Server has instructed a permanent disable.
 *   * RUI_SDK_OPTED_OUT                             Function validation: The application has instructed an opt out.
 *   * RUI_INVALID_PROXY_CREDENTIALS                 Parameter validation: The proxy username and password are not an allowable combination.
 *   * RUI_INVALID_PROXY_PORT                        Parameter validation: The proxy port was not valid.
 *   * RUI_CONFIG_NOT_CREATED                        Function validation: Configuration has not been successfully created.
 *   * RUI_SDK_ALREADY_STOPPED                       Function validation: SDK has already been successfully stopped.
 */
-(RUIRESULTOBJC)setProxyAddress:(NSString*)address
                           port:(uint16_t)port
                       username:(NSString*)username
                       password:(NSString*)password;
    

	/**
	* Sets ReachOutOnAutoSync setting. 
	*
	* setReachOutOnAutoSync() can be called between createConfig() and stopSDK(), and can be called zero or more times.
	*
	* setReachOutOnAutoSync() is a synchronous function, returning when all functionality is completed.
	*
	* @param[in] reachOutOnAutoSyncSetting              Value to enable (true) or disable(false) Reach Out on Auto Sync.
	* @returns   One of the result codes in RUIResult, RUISDKDefines.h:
	*   * RUI_OK                                        Function (which may be synchronous or asynchronous), fully successful during synchronous functionality.
	*   * RUI_SDK_INTERNAL_ERROR_FATAL                  Error:  Irrecoverable internal fatal error.  No further API calls should be made.
	*   * RUI_SDK_ABORTED                               Error:  A required New Registration has failed, and hence the SDK is aborted.  stopSDK and RUISDK destructor are possible.
	*   * RUI_SDK_SUSPENDED                             Function validation: The RUI Server has instructed a temporary back-off.
	*   * RUI_SDK_PERMANENTLY_DISABLED                  Function validation: The RUI Server has instructed a permanent disable.
	*   * RUI_SDK_OPTED_OUT                             Function validation: The application has instructed an opt out.
	*   * RUI_CONFIG_NOT_CREATED                        Function validation: Configuration has not been successfully created.
	*   * RUI_SDK_ALREADY_STOPPED                       Function validation: SDK has already been successfully stopped.
	*/
-(RUIRESULTOBJC)setReachOutOnAutoSync: (BOOL)reachOutOnAutoSyncSetting;

/**
 * Sets or clears the product data.  NOTE:  The product data must be set every time the RUI SDK instance is run.  This is
 * different than V4 of the RUI (Trackerbird) SDK where the supplied product data was stored in the SDK configuration file and
 * if it was not supplied, the values in the configuration file were used.
 *
 * setProductEdition:productLanguage:productVersion:productBuildNumber:() can be called between
 * createConfig:productID:appName:serverURL:protocol:aesKeyHex:multiSessionEnabled:reachOutOnAutoSync:() and stopSDK:() and can
 * be called zero or more times.
 *
 * setProductEdition:productLanguage:productVersion:productBuildNumber:() is a synchronous function returning when all functionality
 * is completed.
 *
 * @see setProductEdition:()
 * @see setProductLanguage:()
 * @see setProductVersion:()
 * @see setProductBuildNumber:()
 * @param[in] productEdition      The product edition to use in RUI Server reports; empty clears the value.
 * @param[in] productLanguage     The product language to use in RUI Server reports; empty clears the value.
 * @param[in] productVersion      The product version to use in RUI Server reports; empty clears the value.
 * @param[in] productBuildNumber  The product build number to use in RUI Server reports; empty clears the value.
 * @returns   One of the result codes in RUISDKDefines.h:
 *   * RUI_OK                                        Function (which may be synchronous or asynchronous), fully successful during synchronous functionality.
 *   * RUI_SDK_INTERNAL_ERROR_FATAL                  Error:  Irrecoverable internal fatal error.  No further API calls should be made.
 *   * RUI_SDK_ABORTED                               Error:  A required New Registration has failed, and hence the SDK is aborted.  StopSDK and DestroySDK are possible.
 *   * RUI_SDK_SUSPENDED                             Function validation: The RUI Server has instructed a temporary backoff.
 *   * RUI_SDK_PERMANENTLY_DISABLED                  Function validation: The RUI Server has instructed a permanent disable.
 *   * RUI_SDK_OPTED_OUT                             Function validation: The application has instructed an opt out.
 *   * RUI_CONFIG_NOT_CREATED                        Function validation: Configuration has not been successfully created.
 *   * RUI_SDK_ALREADY_STOPPED                       Function validation: SDK has already been successfully stopped.
 */
-(RUIRESULTOBJC)setProductEdition:(NSString*)productEdition
                  productLanguage:(NSString*)productLanguage
                   productVersion:(NSString*)productVersion
               productBuildNumber:(NSString*)productBuildNumber;

/**
 * Sets or clears the product data.  NOTE: The product data must be set every time the RUI SDK instance is run.  This is
 * different than V4 of the RUI (Trackerbird) SDK where the supplied product data was stored in the SDK configuration file and
 * if it was not supplied, the values in the configuration file were used.
 *
 * setProductEdition:() can be called between createConfig:productID:appName:serverURL:protocol:aesKeyHex:multiSessionEnabled:reachOutOnAutoSync:()
 * and stopSDK:(), and can be called zero or more times.
 *
 * setProductEdition:() is a synchronous function, returning when all functionality is completed.
 *
 * @see setProductEdition:productLanguage:productVersion:productBuildNumber:()
 * @param[in] productEdition      The product edition to use in RUI Server reports; empty clears the value.
 * @returns   One of the result codes in RUISDKDefines.h:
 *   * RUI_OK                                        Function (which may be synchronous or asynchronous), fully successful during synchronous functionality.
 *   * RUI_SDK_INTERNAL_ERROR_FATAL                  Error:  Irrecoverable internal fatal error.  No further API calls should be made.
 *   * RUI_SDK_ABORTED                               Error:  A required New Registration has failed, and hence the SDK is aborted.  StopSDK and DestroySDK are possible.
 *   * RUI_SDK_SUSPENDED                             Function validation: The RUI Server has instructed a temporary backoff.
 *   * RUI_SDK_PERMANENTLY_DISABLED                  Function validation: The RUI Server has instructed a permanent disable.
 *   * RUI_SDK_OPTED_OUT                             Function validation: The application has instructed an opt out.
 *   * RUI_CONFIG_NOT_CREATED                        Function validation: Configuration has not been successfully created.
 *   * RUI_SDK_ALREADY_STOPPED                       Function validation: SDK has already been successfully stopped.
 */
-(RUIRESULTOBJC)setProductEdition:(NSString*)productEdition;

/**
 * Sets or clears the product data.  NOTE: The product data must be set every time the RUI SDK instance is run.  This is
 * different than V4 of the RUI (Trackerbird) SDK where the supplied product data was stored in the SDK configuration file and
 * if it was not supplied, the values in the configuration file were used.
 *
 * setProductLanguage:() can be called between createConfig:productID:appName:serverURL:protocol:aesKeyHex:multiSessionEnabled:reachOutOnAutoSync:()
 * and stopSDK:(), and can be called zero or more times.
 *
 * setProductLanguage:() is a synchronous function, returning when all functionality is completed.
 *
 * @see setProductEdition:productLanguage:productVersion:productBuildNumber:()
 * @param[in] productLanguage     The product language to use in RUI Server reports; empty clears the value.
 * @returns   One of the result codes in RUISDKDefines.h:
 *   * RUI_OK                                        Function (which may be synchronous or asynchronous), fully successful during synchronous functionality.
 *   * RUI_SDK_INTERNAL_ERROR_FATAL                  Error:  Irrecoverable internal fatal error.  No further API calls should be made.
 *   * RUI_SDK_ABORTED                               Error:  A required New Registration has failed, and hence the SDK is aborted.  StopSDK and DestroySDK are possible.
 *   * RUI_SDK_SUSPENDED                             Function validation: The RUI Server has instructed a temporary backoff.
 *   * RUI_SDK_PERMANENTLY_DISABLED                  Function validation: The RUI Server has instructed a permanent disable.
 *   * RUI_SDK_OPTED_OUT                             Function validation: The application has instructed an opt out.
 *   * RUI_CONFIG_NOT_CREATED                        Function validation: Configuration has not been successfully created.
 *   * RUI_SDK_ALREADY_STOPPED                       Function validation: SDK has already been successfully stopped.
 */
-(RUIRESULTOBJC)setProductLanguage:(NSString*)productLanguage;

/**
 * Sets or clears the product data.  NOTE: The product data must be set every time the RUI SDK instance is run.  This is
 * different than V4 of the RUI (Trackerbird) SDK where the supplied product data was stored in the SDK configuration file and
 * if it was not supplied, the values in the configuration file were used.
 *
 * setProductVersion:() can be called between createConfig:productID:appName:serverURL:protocol:aesKeyHex:multiSessionEnabled:reachOutOnAutoSync:()
 * and stopSDK:(), and can be called zero or more times.
 *
 * setProductVersion:() is a synchronous function, returning when all functionality is completed.
 *
 * @see setProductEdition:productLanguage:productVersion:productBuildNumber:()
 * @param[in] productVersion      The product version to use in RUI Server reports; empty clears the value.
 * @returns   One of the result codes in RUISDKDefines.h:
 *   * RUI_OK                                        Function (which may be synchronous or asynchronous), fully successful during synchronous functionality.
 *   * RUI_SDK_INTERNAL_ERROR_FATAL                  Error:  Irrecoverable internal fatal error.  No further API calls should be made.
 *   * RUI_SDK_ABORTED                               Error:  A required New Registration has failed, and hence the SDK is aborted.  StopSDK and DestroySDK are possible.
 *   * RUI_SDK_SUSPENDED                             Function validation: The RUI Server has instructed a temporary backoff.
 *   * RUI_SDK_PERMANENTLY_DISABLED                  Function validation: The RUI Server has instructed a permanent disable.
 *   * RUI_SDK_OPTED_OUT                             Function validation: The application has instructed an opt out.
 *   * RUI_CONFIG_NOT_CREATED                        Function validation: Configuration has not been successfully created.
 *   * RUI_SDK_ALREADY_STOPPED                       Function validation: SDK has already been successfully stopped.
 */
-(RUIRESULTOBJC)setProductVersion:(NSString*)productVersion;

/**
 * Sets or clears the product data.  NOTE: The product data must be set every time the RUI SDK instance is run.  This is
 * different than V4 of the RUI (Trackerbird) SDK where the supplied product data was stored in the SDK configuration file and
 * if it was not supplied, the values in the configuration file were used.
 *
 * setProductBuildNumber:() can be called between createConfig:productID:appName:serverURL:protocol:aesKeyHex:multiSessionEnabled:reachOutOnAutoSync:()
 * and stopSDK:(), and can be called zero or more times.
 *
 * setProductBuildNumber:() is a synchronous function, returning when all functionality is completed.
 *
 * @see setProductEdition:productLanguage:productVersion:productBuildNumber:()
 * @param[in] productBuildNumber  The product build number to use in RUI Server reports; empty clears the value.
 * @returns   One of the result codes in RUISDKDefines.h:
 *   * RUI_OK                                        Function (which may be synchronous or asynchronous), fully successful during synchronous functionality.
 *   * RUI_SDK_INTERNAL_ERROR_FATAL                  Error:  Irrecoverable internal fatal error.  No further API calls should be made.
 *   * RUI_SDK_ABORTED                               Error:  A required New Registration has failed, and hence the SDK is aborted.  StopSDK and DestroySDK are possible.
 *   * RUI_SDK_SUSPENDED                             Function validation: The RUI Server has instructed a temporary backoff.
 *   * RUI_SDK_PERMANENTLY_DISABLED                  Function validation: The RUI Server has instructed a permanent disable.
 *   * RUI_SDK_OPTED_OUT                             Function validation: The application has instructed an opt out.
 *   * RUI_CONFIG_NOT_CREATED                        Function validation: Configuration has not been successfully created.
 *   * RUI_SDK_ALREADY_STOPPED                       Function validation: SDK has already been successfully stopped.
 */
-(RUIRESULTOBJC)setProductBuildNumber:(NSString*)productBuildNumber;

/**
 * Sets or clears the license data.  The legal parameter values include RUI_KEY_STATUS_UNCHANGED (-1).  NOTE:  Different from
 * the V4 of the RUI SDK, a sessionID parameter can be supplied.
 *
 * setLicenseDataKeyType:keyExpired:keyActivated:keyBlocked:keyAllowed:sessionID:() can be called between
 * createConfig:productID:appName:serverURL:protocol:aesKeyHex:multiSessionEnabled:reachOutOnAutoSync:()
 * and stopSDK:() and can be called zero or more times.  However,
 * the usage requirements of the sessionID parameter are different if
 * setLicenseDataKeyType:keyExpired:keyActivated:keyBlocked:keyAllowed:sessionID:() is called before startSDK() or
 * called after startSDK():
 *   * Before startSDK() regardless of multiSessionEnabled - sessionID must be empty.
 *   * After  startSDK() and multiSessionEnabled = false   - sessionID must be empty.  This is similar to event tracking APIs.
 *   * After  startSDK() and multiSessionEnabled = true    - sessionID must be a current valid value used in startSession:(), or
 *             it can be empty.  This is different than normal event tracking APIs, whereby a empty value is not be allowed.
 *
 * setLicenseDataKeyType:keyExpired:keyActivated:keyBlocked:keyAllowed:sessionID:() can be called while a New Registration
 * is being performed (createConfig:productID:appName:serverURL:protocol:aesKeyHex:multiSessionEnabled:reachOutOnAutoSync:(),
 * startSDK()).  However, the event data is not written to the log file until the New Registration completes, and if the New
 * Registration fails, the data will be lost.
 *
 * setLicenseDataKeyType:keyExpired:keyActivated:keyBlocked:keyAllowed:sessionID:() is an asynchronous function, returning
 * immediately with further functionality executed on separate thread(s).
 *
 * @see checkLicenseKey:returningLicenseArray:()
 * @see setLicenseKey:returningLicenseArray:sessionID:()
 * @param[in] keyType          The key type for the license.
 * @param[in] keyExpired       The key expired flag for the license.
 * @param[in] keyActivated     The key activated flag for the license.
 * @param[in] keyBlocked       The key blocked flag for the license.
 * @param[in] keyAllowed       The key allowed flag for the license.
 * @param[in] sessionID        A session ID complying with above usage (content conditioning and validation rules in startSession:()).
 * @returns   One of the result codes in RUISDKDefines.h:
 *   * RUI_OK                                        Function (which may be synchronous or asynchronous), fully successful during synchronous functionality.
 *   * RUI_SDK_INTERNAL_ERROR_FATAL                  Error:  Irrecoverable internal fatal error.  No further API calls should be made.
 *   * RUI_SDK_ABORTED                               Error:  A required New Registration has failed, and hence the SDK is aborted.  StopSDK and DestroySDK are possible.
 *   * RUI_SDK_SUSPENDED                             Function validation: The RUI Server has instructed a temporary backoff.
 *   * RUI_SDK_PERMANENTLY_DISABLED                  Function validation: The RUI Server has instructed a permanent disable.
 *   * RUI_SDK_OPTED_OUT                             Function validation: The application has instructed an opt out.
 *   * RUI_CONFIG_NOT_CREATED                        Function validation: Configuration has not been successfully created.
 *   * RUI_SDK_ALREADY_STOPPED                       Function validation: SDK has already been successfully stopped.
 *   * RUI_INVALID_SESSION_ID_EXPECTED_EMPTY         Parameter validation: The sessionID is expected to be empty, and it was not.
 *   * RUI_INVALID_SESSION_ID_EXPECTED_NON_EMPTY     Parameter validation: The sessionID is expected to be non-empty, and it was not.
 *   * RUI_INVALID_SESSION_ID_TOO_SHORT              Parameter validation: The sessionID violates its allowable min length.
 *   * RUI_INVALID_SESSION_ID_TOO_LONG               Parameter validation: The sessionID violates its allowable max length.
 *   * RUI_INVALID_SESSION_ID_NOT_ACTIVE             Parameter validation: The sessionID is not currently in use.
 *   * RUI_INVALID_LICENSE_DATA_VALUE                Parameter validation: The license data is not an allowable value.
 */
-(RUIRESULTOBJC)setLicenseDataKeyType:(int32_t)keyType
                           keyExpired:(int32_t)keyExpired
                         keyActivated:(int32_t)keyActivated
                           keyBlocked:(int32_t)keyBlocked
                           keyAllowed:(int32_t)keyAllowed
                            sessionID:(NSString*)sessionID;

/**
 * Sets or clears the custom property data.  NOTE: The custom property data must be set every time the RUI SDK instance is run.
 * This is different than V4 of the RUI SDK where the supplied product data was stored in the RUI SDK configuration file and if
 * it was not supplied, the values in the configuration file were used.
 *
 * setCustomProperty:value:() can be called between createConfig:productID:appName:serverURL:protocol:aesKeyHex:multiSessionEnabled:reachOutOnAutoSync:()
 * and stopSDK:(), and can be called zero or more times.
 *
 * setCustomProperty:value:() is a synchronous function, returning when all functionality is completed.
 *
 * @see setProductEdition:productLanguage:productVersion:productBuildNumber:()
 * @param[in] customPropertyID The 1-based index of the custom property to set.
 * @param[in] customValue      The custom property value to use in RUI Server reports; empty clears the value.
 * @returns   One of the result codes in RUISDKDefines.h:
 *   * RUI_OK                                        Function (which may be synchronous or asynchronous), fully successful during synchronous functionality.
 *   * RUI_SDK_INTERNAL_ERROR_FATAL                  Error:  Irrecoverable internal fatal error.  No further API calls should be made.
 *   * RUI_SDK_ABORTED                               Error:  A required New Registration has failed, and hence the SDK is aborted.  StopSDK and DestroySDK are possible.
 *   * RUI_SDK_SUSPENDED                             Function validation: The RUI Server has instructed a temporary backoff.
 *   * RUI_SDK_PERMANENTLY_DISABLED                  Function validation: The RUI Server has instructed a permanent disable.
 *   * RUI_SDK_OPTED_OUT                             Function validation: The application has instructed an opt out.
 *   * RUI_CONFIG_NOT_CREATED                        Function validation: Configuration has not been successfully created.
 *   * RUI_SDK_ALREADY_STOPPED                       Function validation: SDK has already been successfully stopped.
 *   * RUI_INVALID_CUSTOM_PROPERTY_ID                Parameter validation: The 1-based customPropertyID violates its allowable range.
 */
-(RUIRESULTOBJC)setCustomProperty:(uint32_t)customPropertyID
                            value:(NSString*)customValue;

/**
 * Starts the RUI SDK.  startSDK() must be paired with a call to stopSDK:().  After the RUI SDK is started, the various
 * event tracking APIs are available to be used.  If createConfig() did not detect a configuration file, startSDK()
 * will perform a New Registration with the RUI Server.  Until a New Registration is complete, the RUI SDK will not be able save
 * event data to a log file or perform synchronization with the RUI Server.  A successful New Registration (or presence of a
 * configuration file) will put the RUI SDK into a normal running state, whereby events are saved to a log file, automatic and
 * manual synchronizations with the RUI Server are possible, and ReachOut™s from the Server are possible.  A failed New
 * Registration will put the RUI SDK into an aborted state, not allowing further activity.
 *
 * startSDK() must be called after createConfig:productID:appName:serverURL:protocol:aesKeyHex:multiSessionEnabled:reachOutOnAutoSync:(),
 * and must be called only once.
 *
 * startSDK() is an asynchronous function, returning immediately with further functionality executed on separate thread(s).
 *
 * @see getState()
 * @returns   One of the result codes in RUISDKDefines.h:
 *   * RUI_OK                                        Function (which may be synchronous or asynchronous), fully successful during synchronous functionality.
 *   * RUI_SDK_INTERNAL_ERROR_FATAL                  Error:  Irrecoverable internal fatal error.  No further API calls should be made.
 *   * RUI_SDK_ABORTED                               Error:  A required New Registration has failed, and hence the SDK is aborted.  StopSDK and DestroySDK are possible.
 *   * RUI_SDK_PERMANENTLY_DISABLED                  Function validation: The RUI Server has instructed a permanent disable.
 *   * RUI_CONFIG_NOT_CREATED                        Function validation: Configuration has not been successfully created.
 *   * RUI_SDK_ALREADY_STARTED                       Function validation: SDK has already been successfully started.
 *   * RUI_SDK_ALREADY_STOPPED                       Function validation: SDK has already been successfully stopped.
 */
-(RUIRESULTOBJC)startSDK;

/**
 * Stops the RUI SDK that was started with startSDK().  If explicit sessions are allowed (multiSessionsEnabled = true in
 * createConfig:productID:appName:serverURL:protocol:aesKeyHex:multiSessionEnabled:reachOutOnAutoSync:()), then any sessions
 * that have been started with startSession:() that have not been stopped with
 * stopSession:() are automatically stopped.  A manual synchronization with the RUI Server, sync:(), will be performed at
 * stop depending on the value of doSync:
 *   * -1 : Do not perform a manual synchronization with the RUI Server as part of the stop.
 *   *  0 : Perform a manual synchronization with the RUI Server as part of the stop; wait indefinitely for completion.
 *   * >0 : Perform a manual synchronization with the RUI Server as part of the stop; wait only doSync seconds for completion.
 *
 * stopSDK:() must be called after startSDK() and must be called only once.  After stopSDK:() is called, the various
 * event tracking APIs are no longer available.  The only APIs available are getState() and the destructor.  The RUI
 * SDK cannot be re-started with a subsequent call to startSDK().
 *
 * stopSDK:() is a synchronous function, including the manual synchronization with the RUI Server (if requested), returning
 * when all functionality is completed.
 *
 * @see createConfig:productID:appName:serverURL:protocol:aesKeyHex:multiSessionEnabled:reachOutOnAutoSync:()
 * @see startSDK()
 * @see sync:()
 * @param[in] doSync       Indicates whether to do a manual synchronization as part of the stop, and if so, the wait limit.
 * @returns   One of the result codes in RUISDKDefines.h:
 *   * RUI_OK                                        Function (which may be synchronous or asynchronous), fully successful during synchronous functionality.
 *   * RUI_SDK_INTERNAL_ERROR_FATAL                  Error:  Irrecoverable internal fatal error.  No further API calls should be made.
 *   * RUI_SDK_ABORTED                               Error:  A required New Registration has failed, and hence the SDK is aborted.  StopSDK and DestroySDK are possible.
 *   * RUI_CONFIG_NOT_CREATED                        Function validation: Configuration has not been successfully created.
 *   * RUI_SDK_NOT_STARTED                           Function validation: SDK has not been successfully started.
 *   * RUI_SDK_ALREADY_STOPPED                       Function validation: SDK has already been successfully stopped.
 *   * RUI_INVALID_DO_SYNC_VALUE                     Parameter validation: The doSync manual sync flag/limit violates its allowable range.
 */
-(RUIRESULTOBJC)stopSDK:(int32_t)doSync;

/**
 * Performs a manual synchronization with the RUI Server.  The RUI SDK periodically performs automatic synchronizations with
 * the RUI Server.  sync:() provides the client an ability to explicitly synchronize with the RUI Server at a specific time.
 * The manual synchronization can request a ReachOut™ with getReachOut.  NOTE:  Similar to reachOutOnAutoSync
 * (createConfig:productID:appName:serverURL:protocol:aesKeyHex:multiSessionEnabled:reachOutOnAutoSync:()),
 * the ReachOut™ will not be requested if there is no registered handler (initRegisterDefaultGraphicalReachOutHandler:() and
 * setReachOutHandler:()).
 *
 * sync:() can be called between startSDK() and stopSDK:() and can be called zero or more times.  NOTE: sync:() will
 * not be successful if a New Registration is in progress (i.e.
 * createConfig:productID:appName:serverURL:protocol:aesKeyHex:multiSessionEnabled:reachOutOnAutoSync:() and startSDK()).
 * A manual synchronization with the RUI Server can be associated with stopSDK:().
 *
 * sync:() is an asynchronous function returning immediately with further functionality executed on separate thread(s).
 *
 * @see initRegisterDefaultGraphicalReachOutHandler:()
 * @see createConfig:productID:appName:serverURL:protocol:aesKeyHex:multiSessionEnabled:reachOutOnAutoSync:()
 * @see setReachOutHandler:()
 * @param[in] getReachOut  Indicates whether a ReachOut™ should be requested from the RUI Server.
 * @returns   One of the result codes in RUISDKDefines.h:
 *   * RUI_OK                                        Function (which may be synchronous or asynchronous), fully successful during synchronous functionality.
 *   * RUI_SDK_INTERNAL_ERROR_FATAL                  Error:  Irrecoverable internal fatal error.  No further API calls should be made.
 *   * RUI_SDK_ABORTED                               Error:  A required New Registration has failed, and hence the SDK is aborted.  StopSDK and DestroySDK are possible.
 *   * RUI_SDK_SUSPENDED                             Function validation: The RUI Server has instructed a temporary backoff.
 *   * RUI_SDK_PERMANENTLY_DISABLED                  Function validation: The RUI Server has instructed a permanent disable.
 *   * RUI_SDK_OPTED_OUT                             Function validation: The application has instructed an opt out.
 *   * RUI_CONFIG_NOT_CREATED                        Function validation: Configuration has not been successfully created.
 *   * RUI_SDK_NOT_STARTED                           Function validation: SDK has not been successfully started.
 *   * RUI_SYNC_ALREADY_RUNNING                      Function validation: A sync with the RUI Server is already running.
 *   * RUI_SDK_ALREADY_STOPPED                       Function validation: SDK has already been successfully stopped.
 *   * RUI_TIME_THRESHOLD_NOT_REACHED                Function validation: The API call frequency threshold (set by the RUI Server) has not been reached.
 */
-(RUIRESULTOBJC)sync:(BOOL)getReachOut;

/**
 * Starts an explicit session for event tracking in the RUI SDK.  Must be paired with a call to stopSession:().  Explicit
 * sessions are allowed only if createConfig:productID:appName:serverURL:protocol:aesKeyHex:multiSessionEnabled:reachOutOnAutoSync:()
 * was called with multiSessionEnabled = true.  When explicit sessions are enabled, a valid sessionID becomes a required
 * parameter to the event tracking APIs.
 *
 * The content of a sessionID is conditioned and validated (after conditioning) with the following rules:
 *   * Conditioning: All leading white space is removed.
 *   * Conditioning: All trailing white space is removed.
 *   * Conditioning: All internal white space is removed, except spaces (' ') are left as is.
 *   * Validation: Cannot be shorter than 10 UTF8 characters.
 *   * Validation: Cannot be longer than 64 UTF8 characters.
 *
 * The resulting conditioned and validated sessionID must be unique (i.e. not already in use).  NOTE: With the above
 * conditioning, two sessionID's that differ only by white space or after the 64th character, will not be unique.  A sessionID
 * should not be re-used for different sessions.
 *
 * startSession:() can be called between startSDK() and stopSDK:(), and can be called zero or more times.
 *
 * startSession:() is a synchronous function, returning when all functionality is completed.
 *
 * @see createConfig:productID:appName:serverURL:protocol:aesKeyHex:multiSessionEnabled:reachOutOnAutoSync:()
 * @see stopSession:()
 * @param[in] sessionID    The client-created valid and unique session ID to be registered as a new session start.
 * @returns   One of the result codes in RUISDKDefines.h:
 *   * RUI_OK                                        Function (which may be synchronous or asynchronous), fully successful during synchronous functionality.
 *   * RUI_SDK_INTERNAL_ERROR_FATAL                  Error:  Irrecoverable internal fatal error.  No further API calls should be made.
 *   * RUI_SDK_ABORTED                               Error:  A required New Registration has failed, and hence the SDK is aborted.  StopSDK and DestroySDK are possible.
 *   * RUI_SDK_SUSPENDED                             Function validation: The RUI Server has instructed a temporary backoff.
 *   * RUI_SDK_PERMANENTLY_DISABLED                  Function validation: The RUI Server has instructed a permanent disable.
 *   * RUI_SDK_OPTED_OUT                             Function validation: The application has instructed an opt out.
 *   * RUI_CONFIG_NOT_CREATED                        Function validation: Configuration has not been successfully created.
 *   * RUI_SDK_NOT_STARTED                           Function validation: SDK has not been successfully started.
 *   * RUI_FUNCTION_NOT_AVAIL                        Function validation: Function is not available.
 *   * RUI_SDK_ALREADY_STOPPED                       Function validation: SDK has already been successfully stopped.
 *   * RUI_INVALID_SESSION_ID_EXPECTED_NON_EMPTY     Parameter validation: The sessionID is expected to be non-empty, and it was not.
 *   * RUI_INVALID_SESSION_ID_TOO_SHORT              Parameter validation: The sessionID violates its allowable min length.
 *   * RUI_INVALID_SESSION_ID_TOO_LONG               Parameter validation: The sessionID violates its allowable max length.
 *   * RUI_INVALID_SESSION_ID_ALREADY_ACTIVE         Parameter validation: The sessionID is already currently in use.
 */
-(RUIRESULTOBJC)startSession:(NSString*)sessionID;

/**
 * Stops an explicit session started with startSession:().  Explicit sessions are allowed only if
 * createConfig:productID:appName:serverURL:protocol:aesKeyHex:multiSessionEnabled:reachOutOnAutoSync:() was called with
 * multiSessionEnabled = true.  Any explicit sessions not ended with a call to stopSession:() are automatically ended when
 * stopSDK:() is called.
 *
 * stopSession:() can be called between startSDK() and stopSDK:(), and can be called zero or more times.
 *
 * stopSession:() is a synchronous function, returning when all functionality is completed.
 *
 * @see createConfig:productID:appName:serverURL:protocol:aesKeyHex:multiSessionEnabled:reachOutOnAutoSync:()
 * @see startSession:()
 * @see stopSDK:()
 * @param[in] sessionID    A current valid session ID to be stopped (content conditioning and validation rules in startSession:()).
 * @returns   One of the result codes in RUISDKDefines.h:
 *   * RUI_OK                                        Function (which may be synchronous or asynchronous), fully successful during synchronous functionality.
 *   * RUI_SDK_INTERNAL_ERROR_FATAL                  Error:  Irrecoverable internal fatal error.  No further API calls should be made.
 *   * RUI_SDK_ABORTED                               Error:  A required New Registration has failed, and hence the SDK is aborted.  StopSDK and DestroySDK are possible.
 *   * RUI_SDK_SUSPENDED                             Function validation: The RUI Server has instructed a temporary backoff.
 *   * RUI_SDK_PERMANENTLY_DISABLED                  Function validation: The RUI Server has instructed a permanent disable.
 *   * RUI_SDK_OPTED_OUT                             Function validation: The application has instructed an opt out.
 *   * RUI_CONFIG_NOT_CREATED                        Function validation: Configuration has not been successfully created.
 *   * RUI_SDK_NOT_STARTED                           Function validation: SDK has not been successfully started.
 *   * RUI_FUNCTION_NOT_AVAIL                        Function validation: Function is not available.
 *   * RUI_SDK_ALREADY_STOPPED                       Function validation: SDK has already been successfully stopped.
 *   * RUI_INVALID_SESSION_ID_EXPECTED_NON_EMPTY     Parameter validation: The sessionID is expected to be non-empty, and it was not.
 *   * RUI_INVALID_SESSION_ID_TOO_SHORT              Parameter validation: The sessionID violates its allowable min length.
 *   * RUI_INVALID_SESSION_ID_TOO_LONG               Parameter validation: The sessionID violates its allowable max length.
 *   * RUI_INVALID_SESSION_ID_NOT_ACTIVE             Parameter validation: The sessionID is not currently in use.
 */
-(RUIRESULTOBJC)stopSession:(NSString*)sessionID;

/**
 * Logs an exception event with the supplied data.  NOTE:  Different than V4 of the RUI (Trackerbird) SDK, trackException:sessionID:()
 * accepts a sessionID parameter.  The usage requirements of the sessionID parameter are the following:
 *   * multiSessionEnabled = false - sessionID must be empty.
 *   * multiSessionEnabled = true  - sessionID must be a current valid value used in startSession:(), or it can be empty.
 *             This is different than normal event tracking APIs, whereby an empty value is not allowed.
 *
 * The content of exceptionData.className and exceptionData.methodName is conditioned and validated (after conditioning) with
 * the following rules:
 *   * Conditioning: All leading white space is removed.
 *   * Conditioning: All trailing white space is removed.
 *   * Conditioning: All internal white space is removed, except spaces (' ') are left as is.
 *   * Conditioning: Trimmed to at most 128 UTF8 characters.
 *   * Validation: Cannot be empty.
 *
 * trackException:sessionID:() can be called between startSDK() and stopSDK:(), and can be called zero or more times.
 *
 * trackException:sessionID:() can be called while a New Registration is being performed
 * (createConfig:productID:appName:serverURL:protocol:aesKeyHex:multiSessionEnabled:reachOutOnAutoSync:(), startSDK()).  However,
 * the event data is not written to the log file until the New Registration completes, and if the New Registration fails, the
 * data will be lost.
 *
 * trackException:sessionID:() is an asynchronous function, returning immediately with further functionality executed on separate thread(s).
 *
 * @see createConfig:productID:appName:serverURL:protocol:aesKeyHex:multiSessionEnabled:reachOutOnAutoSync:()
 * @see startSDK()
 * @see startSession:()
 * @param[in] exceptionData  The exception data to log:
 *                           * className  - Class catching exception.  Content conditioning and validation rules above.
 *                           * methodName - Method catching exception.  Content conditioning and validation rules above.
 *                           * message    - Exception message.  No content restrictions, but trimmed to a maximum length determined
 *                                          by the RUI Server.
 *                           * stackTrace - Exception stack trace.  No content restrictions, but trimmed to a maximum length
 *                                        - determined by the RUI Server.
 * @param[in] sessionID      A session ID complying with above usage (content conditioning and validation rules in startSession:()).
 * @returns   One of the result codes in RUISDKDefines.h:
 *   * RUI_OK                                        Function (which may be synchronous or asynchronous), fully successful during synchronous functionality.
 *   * RUI_SDK_INTERNAL_ERROR_FATAL                  Error:  Irrecoverable internal fatal error.  No further API calls should be made.
 *   * RUI_SDK_ABORTED                               Error:  A required New Registration has failed, and hence the SDK is aborted.  StopSDK and DestroySDK are possible.
 *   * RUI_SDK_SUSPENDED                             Function validation: The RUI Server has instructed a temporary backoff.
 *   * RUI_SDK_PERMANENTLY_DISABLED                  Function validation: The RUI Server has instructed a permanent disable.
 *   * RUI_SDK_OPTED_OUT                             Function validation: The application has instructed an opt out.
 *   * RUI_CONFIG_NOT_CREATED                        Function validation: Configuration has not been successfully created.
 *   * RUI_SDK_NOT_STARTED                           Function validation: SDK has not been successfully started.
 *   * RUI_SDK_ALREADY_STOPPED                       Function validation: SDK has already been successfully stopped.
 *   * RUI_INVALID_SESSION_ID_EXPECTED_EMPTY         Parameter validation: The sessionID is expected to be empty, and it was not.
 *   * RUI_INVALID_SESSION_ID_TOO_SHORT              Parameter validation: The sessionID violates its allowable min length.
 *   * RUI_INVALID_SESSION_ID_TOO_LONG               Parameter validation: The sessionID violates its allowable max length.
 *   * RUI_INVALID_SESSION_ID_NOT_ACTIVE             Parameter validation: The sessionID is not currently in use.
 *   * RUI_INVALID_PARAMETER_EXPECTED_NON_EMPTY      Parameter validation: Some API parameter is expected to be non-empty, and is not.
 */
-(RUIRESULTOBJC)trackException:(RUIExceptionEventOBJC*)event
                     sessionID:(NSString*)sessionID;

/**
 * Logs a normal event with the supplied data.   The usage requirements of the sessionID parameter are the following:
 *   * multiSessionEnabled = false - sessionID must be empty.
 *   * multiSessionEnabled = true  - sessionID must be a current valid value used in startSession:().
 *
 * NOTE:  Unlike V4 of the RUI SDK, there is no concept of extended names (for eventCategory and eventName).  The content of
 * eventCategory and eventName is conditioned and validated (after conditioning) with the following rules:
 *   * Conditioning: All leading white space is removed.
 *   * Conditioning: All trailing white space is removed.
 *   * Conditioning: All internal white space is removed, except spaces (' ') are left as is.
 *   * Conditioning: Trimmed to at most 128 UTF8 characters.
 *   * Validation: eventCategory can be empty; eventName cannot be empty.
 *
 * trackEventCategory:eventName:sessionID:() can be called between startSDK() and stopSDK:(), and can be called zero or more times.
 *
 * trackEventCategory:eventName:sessionID:() can be called while a New Registration is being performed
 * (createConfig:productID:appName:serverURL:protocol:aesKeyHex:multiSessionEnabled:reachOutOnAutoSync:(), startSDK()).  However,
 * the event data is not written to the log file until the New Registration completes, and if the New Registration fails, the
 * data will be lost.
 *
 * trackEventCategory:eventName:sessionID:() is an asynchronous function, returning immediately with further functionality executed
 * on separate thread(s).
 *
 * @see createConfig:productID:appName:serverURL:protocol:aesKeyHex:multiSessionEnabled:reachOutOnAutoSync:()
 * @see startSDK()
 * @see startSession:()
 * @param[in] eventCategory  Category of the event with content conditioning and validation rules above.
 * @param[in] eventName      Name of the event with content conditioning and validation rules above.
 * @param[in] sessionID      A session ID complying with above usage (content conditioning and validation rules in startSession:()).
 * @returns   One of the result codes in RUISDKDefines.h:
 *   * RUI_OK                                        Function (which may be synchronous or asynchronous), fully successful during synchronous functionality.
 *   * RUI_SDK_INTERNAL_ERROR_FATAL                  Error:  Irrecoverable internal fatal error.  No further API calls should be made.
 *   * RUI_SDK_ABORTED                               Error:  A required New Registration has failed, and hence the SDK is aborted.  StopSDK and DestroySDK are possible.
 *   * RUI_SDK_SUSPENDED                             Function validation: The RUI Server has instructed a temporary backoff.
 *   * RUI_SDK_PERMANENTLY_DISABLED                  Function validation: The RUI Server has instructed a permanent disable.
 *   * RUI_SDK_OPTED_OUT                             Function validation: The application has instructed an opt out.
 *   * RUI_CONFIG_NOT_CREATED                        Function validation: Configuration has not been successfully created.
 *   * RUI_SDK_NOT_STARTED                           Function validation: SDK has not been successfully started.
 *   * RUI_SDK_ALREADY_STOPPED                       Function validation: SDK has already been successfully stopped.
 *   * RUI_INVALID_SESSION_ID_EXPECTED_EMPTY         Parameter validation: The sessionID is expected to be empty, and it was not.
 *   * RUI_INVALID_SESSION_ID_EXPECTED_NON_EMPTY     Parameter validation: The sessionID is expected to be non-empty, and it was not.
 *   * RUI_INVALID_SESSION_ID_TOO_SHORT              Parameter validation: The sessionID violates its allowable min length.
 *   * RUI_INVALID_SESSION_ID_TOO_LONG               Parameter validation: The sessionID violates its allowable max length.
 *   * RUI_INVALID_SESSION_ID_NOT_ACTIVE             Parameter validation: The sessionID is not currently in use.
 *   * RUI_INVALID_PARAMETER_EXPECTED_NON_EMPTY      Parameter validation: Some API parameter is expected to be non-empty, and is not.
 */
-(RUIRESULTOBJC)trackEventCategory:(NSString*)eventCategory
                         eventName:(NSString*)eventName
                         sessionID:(NSString*)sessionID;

/**
 * Logs a normal event with the supplied data, including a custom numeric field.   The usage requirements of the sessionID
 * parameter are the following:
 *   * multiSessionEnabled = false - sessionID must be empty.
 *   * multiSessionEnabled = true  - sessionID must be a current valid value used in startSession:().
 *
 * NOTE:  Unlike V4 of the RUI SDK, there is no concept of extended names (for eventCategory and eventName).  The content of
 * eventCategory and eventName is conditioned and validated (after conditioning) with the following rules:
 *   * Conditioning: All leading white space is removed.
 *   * Conditioning: All trailing white space is removed.
 *   * Conditioning: All internal white space is removed, except spaces (' ') are left as is.
 *   * Conditioning: Trimmed to at most 128 UTF8 characters.
 *   * Validation: eventCategory can be empty; eventName cannot be empty.
 *
 * trackEventCategory:eventName:withNumeric:sessionID:() can be called between startSDK() and stopSDK:(), and can be called zero or more times.
 *
 * trackEventCategory:eventName:withNumeric:sessionID:() can be called while a New Registration is being performed
 * (createConfig:productID:appName:serverURL:protocol:aesKeyHex:multiSessionEnabled:reachOutOnAutoSync:(), startSDK()).  However,
 * the event data is not written to the log file until the New Registration completes, and if the New Registration fails, the
 * data will be lost.
 *
 * trackEventCategory:eventName:withNumeric:sessionID:() is an asynchronous function, returning immediately with further functionality
 * executed on separate thread(s).
 *
 * @see createConfig:productID:appName:serverURL:protocol:aesKeyHex:multiSessionEnabled:reachOutOnAutoSync:()
 * @see startSDK()
 * @see startSession:()
 * @param[in] eventCategory  Category of the event with content conditioning and validation rules above.
 * @param[in] eventName      Name of the event with content conditioning and validation rules above.
 * @param[in] customValue    Custom numeric data associated with the event.
 * @param[in] sessionID      A session ID complying with above usage (content conditioning and validation rules in startSession:()).
 * @returns   One of the result codes in RUISDKDefines.h:
 *   * RUI_OK                                        Function (which may be synchronous or asynchronous), fully successful during synchronous functionality.
 *   * RUI_SDK_INTERNAL_ERROR_FATAL                  Error:  Irrecoverable internal fatal error.  No further API calls should be made.
 *   * RUI_SDK_ABORTED                               Error:  A required New Registration has failed, and hence the SDK is aborted.  StopSDK and DestroySDK are possible.
 *   * RUI_SDK_SUSPENDED                             Function validation: The RUI Server has instructed a temporary backoff.
 *   * RUI_SDK_PERMANENTLY_DISABLED                  Function validation: The RUI Server has instructed a permanent disable.
 *   * RUI_SDK_OPTED_OUT                             Function validation: The application has instructed an opt out.
 *   * RUI_CONFIG_NOT_CREATED                        Function validation: Configuration has not been successfully created.
 *   * RUI_SDK_NOT_STARTED                           Function validation: SDK has not been successfully started.
 *   * RUI_SDK_ALREADY_STOPPED                       Function validation: SDK has already been successfully stopped.
 *   * RUI_INVALID_SESSION_ID_EXPECTED_EMPTY         Parameter validation: The sessionID is expected to be empty, and it was not.
 *   * RUI_INVALID_SESSION_ID_EXPECTED_NON_EMPTY     Parameter validation: The sessionID is expected to be non-empty, and it was not.
 *   * RUI_INVALID_SESSION_ID_TOO_SHORT              Parameter validation: The sessionID violates its allowable min length.
 *   * RUI_INVALID_SESSION_ID_TOO_LONG               Parameter validation: The sessionID violates its allowable max length.
 *   * RUI_INVALID_SESSION_ID_NOT_ACTIVE             Parameter validation: The sessionID is not currently in use.
 *   * RUI_INVALID_PARAMETER_EXPECTED_NON_EMPTY      Parameter validation: Some API parameter is expected to be non-empty, and is not.
 */
-(RUIRESULTOBJC)trackEventCategory:(NSString*)eventCategory
                         eventName:(NSString*)eventName
                       withNumeric:(double)customValue
                         sessionID:(NSString*)sessionID;

/**
 * Logs a normal event with the supplied data, including a custom string field.   The usage requirements of the sessionID
 * parameter are the following:
 *   * multiSessionEnabled = false - sessionID must be empty.
 *   * multiSessionEnabled = true  - sessionID must be a current valid value used in startSession:().
 *
 * NOTE:  Unlike V4 of the RUI SDK, there is no concept of extended names (for eventCategory and eventName).  The content of
 * eventCategory and eventName is conditioned and validated (after conditioning) with the following rules:
 *   * Conditioning: All leading white space is removed.
 *   * Conditioning: All trailing white space is removed.
 *   * Conditioning: All internal white space is removed, except spaces (' ') are left as is.
 *   * Conditioning: Trimmed to at most 128 UTF8 characters.
 *   * Validation: eventCategory can be empty; eventName cannot be empty.
 *
 * trackEventCategory:eventName:withText:sessionID:() can be called between startSDK() and stopSDK:(), and can be called zero or more times.
 *
 * trackEventCategory:eventName:withText:sessionID:() can be called while a New Registration is being performed (i.e.
 * createConfig:productID:appName:serverURL:protocol:aesKeyHex:multiSessionEnabled:reachOutOnAutoSync:(), startSDK()).  However,
 * the event data is not written to the log file until the New Registration completes, and if the New Registration fails the
 * data will be lost.
 *
 * trackEventCategory:eventName:withText:sessionID:() is an asynchronous function, returning immediately with further functionality
 * executed on separate thread(s).
 *
 * @see createConfig:productID:appName:serverURL:protocol:aesKeyHex:multiSessionEnabled:reachOutOnAutoSync:()
 * @see startSDK()
 * @see startSession:()
 * @param[in] eventCategory  Category of the event with content conditioning and validation rules above.
 * @param[in] eventName      Name of the event with content conditioning and validation rules above.
 * @param[in] customValue    Custom text data associated with the event, cannot be empty.  Trimmed to a maximum length determined
 *                           by the RUI Server.
 * @param[in] sessionID      A session ID complying with above usage (content conditioning and validation rules in startSession:()).
 * @returns   One of the result codes in RUISDKDefines.h:
 *   * RUI_OK                                        Function (which may be synchronous or asynchronous), fully successful during synchronous functionality.
 *   * RUI_SDK_INTERNAL_ERROR_FATAL                  Error:  Irrecoverable internal fatal error.  No further API calls should be made.
 *   * RUI_SDK_ABORTED                               Error:  A required New Registration has failed, and hence the SDK is aborted.  StopSDK and DestroySDK are possible.
 *   * RUI_SDK_SUSPENDED                             Function validation: The RUI Server has instructed a temporary backoff.
 *   * RUI_SDK_PERMANENTLY_DISABLED                  Function validation: The RUI Server has instructed a permanent disable.
 *   * RUI_SDK_OPTED_OUT                             Function validation: The application has instructed an opt out.
 *   * RUI_CONFIG_NOT_CREATED                        Function validation: Configuration has not been successfully created.
 *   * RUI_SDK_NOT_STARTED                           Function validation: SDK has not been successfully started.
 *   * RUI_SDK_ALREADY_STOPPED                       Function validation: SDK has already been successfully stopped.
 *   * RUI_INVALID_SESSION_ID_EXPECTED_EMPTY         Parameter validation: The sessionID is expected to be empty, and it was not.
 *   * RUI_INVALID_SESSION_ID_EXPECTED_NON_EMPTY     Parameter validation: The sessionID is expected to be non-empty, and it was not.
 *   * RUI_INVALID_SESSION_ID_TOO_SHORT              Parameter validation: The sessionID violates its allowable min length.
 *   * RUI_INVALID_SESSION_ID_TOO_LONG               Parameter validation: The sessionID violates its allowable max length.
 *   * RUI_INVALID_SESSION_ID_NOT_ACTIVE             Parameter validation: The sessionID is not currently in use.
 *   * RUI_INVALID_PARAMETER_EXPECTED_NON_EMPTY      Parameter validation: Some API parameter is expected to be non-empty, and is not.
 */
-(RUIRESULTOBJC)trackEventCategory:(NSString*)eventCategory
                         eventName:(NSString*)eventName
                          withText:(NSString*)customValue
                         sessionID:(NSString*)sessionID;

/**
 * Logs a normal event with the supplied data, including an array of custom name/value pairs.   The usage requirements of the
 * sessionID parameter are the following:
 *   * multiSessionEnabled = false - sessionID must be empty.
 *   * multiSessionEnabled = true  - sessionID must be a current valid value used in startSession:().
 *
 * NOTE:  Unlike V4 of the RUI SDK, there is no concept of extended names (for eventCategory and eventName).  The content of
 * eventCategory and eventName is conditioned and validated (after conditioning) with the following rules:
 *   * Conditioning: All leading white space is removed.
 *   * Conditioning: All trailing white space is removed.
 *   * Conditioning: All internal white space is removed, except spaces (' ') are left as is.
 *   * Conditioning: Trimmed to at most 128 UTF8 characters.
 *   * Validation: eventCategory can be empty; eventName cannot be empty.
 *
 * trackEventCategory:eventName:withNameValuePairs:sesionID:() can be called between startSDK() and stopSDK:(), and can be called
 * zero or more times.
 *
 * trackEventCategory:eventName:withNameValuePairs:sesionID:() can be called while a New Registration is being performed
 * (createConfig:productID:appName:serverURL:protocol:aesKeyHex:multiSessionEnabled:reachOutOnAutoSync:(), startSDK()).  However,
 * the event data is not written to the log file until the New Registration completes, and if the New Registration fails, the
 * data will be lost.
 *
 * trackEventCategory:eventName:withNameValuePairs:sesionID:() is an asynchronous function, returning immediately with further
 * functionality executed on separate thread(s).
 *
 * @see createConfig:productID:appName:serverURL:protocol:aesKeyHex:multiSessionEnabled:reachOutOnAutoSync:()
 * @see startSDK()
 * @see startSession:()
 * @param[in] eventCategory  Category of the event with content conditioning and validation rules above.
 * @param[in] eventName      Name of the event with content conditioning and validation rules above.
 * @param[in] customValues   Custom data associated with the event, cannot be empty.  A given name and/or value can be
 *                           empty.  A given name cannot contain white space.  All names and values are trimmed to a maximum
 *                           length determined by the RUI Server.
 * @param[in] sessionID      A session ID complying with above usage (content conditioning and validation rules in startSession:()).
 * @returns   One of the result codes in RUISDKDefines.h:
 *   * RUI_OK                                        Function (which may be synchronous or asynchronous), fully successful during synchronous functionality.
 *   * RUI_SDK_INTERNAL_ERROR_FATAL                  Error:  Irrecoverable internal fatal error.  No further API calls should be made.
 *   * RUI_SDK_ABORTED                               Error:  A required New Registration has failed, and hence the SDK is aborted.  StopSDK and DestroySDK are possible.
 *   * RUI_SDK_SUSPENDED                             Function validation: The RUI Server has instructed a temporary backoff.
 *   * RUI_SDK_PERMANENTLY_DISABLED                  Function validation: The RUI Server has instructed a permanent disable.
 *   * RUI_SDK_OPTED_OUT                             Function validation: The application has instructed an opt out.
 *   * RUI_CONFIG_NOT_CREATED                        Function validation: Configuration has not been successfully created.
 *   * RUI_SDK_NOT_STARTED                           Function validation: SDK has not been successfully started.
 *   * RUI_SDK_ALREADY_STOPPED                       Function validation: SDK has already been successfully stopped.
 *   * RUI_INVALID_SESSION_ID_EXPECTED_EMPTY         Parameter validation: The sessionID is expected to be empty, and it was not.
 *   * RUI_INVALID_SESSION_ID_EXPECTED_NON_EMPTY     Parameter validation: The sessionID is expected to be non-empty, and it was not.
 *   * RUI_INVALID_SESSION_ID_TOO_SHORT              Parameter validation: The sessionID violates its allowable min length.
 *   * RUI_INVALID_SESSION_ID_TOO_LONG               Parameter validation: The sessionID violates its allowable max length.
 *   * RUI_INVALID_SESSION_ID_NOT_ACTIVE             Parameter validation: The sessionID is not currently in use.
 *   * RUI_INVALID_PARAMETER_EXPECTED_NON_NULL       Parameter validation: Some API parameter is expected to be non-NULL, and is not.
 *   * RUI_INVALID_PARAMETER_EXPECTED_NON_EMPTY      Parameter validation: Some API parameter is expected to be non-empty, and is not.
 *   * RUI_INVALID_PARAMETER_EXPECTED_NO_WHITESPACE  Parameter validation: Some API parameter is expected to be free of whitespace, and is not.
 */
-(RUIRESULTOBJC)trackEventCategory:(NSString*)eventCategory
                         eventName:(NSString*)eventName
                withNameValuePairs:(NSArray*)customValues
                         sessionID:(NSString*)sessionID;

/**
 * Tests the connection between the RUI SDK and the RUI Server.  If a valid configuration file exists
 * (createConfig:productID:appName:serverURL:protocol:aesKeyHex:multiSessionEnabled:reachOutOnAutoSync:()), the
 * URL used for the test will be the one in that file, set by the RUI Server.  Otherwise, the URL used for the test will be one
 * set by the client in the call to createConfig:productID:appName:serverURL:protocol:aesKeyHex:multiSessionEnabled:reachOutOnAutoSync:().
 * If set, the proxy is used during the test (setProxyAddress:port:username:password:()).
 *
 * testConnection() can be called between createConfig:productID:appName:serverURL:protocol:aesKeyHex:multiSessionEnabled:reachOutOnAutoSync:()
 * and stopSDK:(), and can be called zero or more times.
 *
 * testConnection() is a synchronous function, returning when all functionality is completed.
 *
 * @see createConfig:productID:appName:serverURL:protocol:aesKeyHex:multiSessionEnabled:reachOutOnAutoSync:()
 * @see setProxyAddress:port:username:password:()
 * @returns   One of the result codes in RUISDKDefines.h:
 *   * RUI_OK                                        Function (which may be synchronous or asynchronous), fully successful during synchronous functionality.
 *   * RUI_SDK_INTERNAL_ERROR_FATAL                  Error:  Irrecoverable internal fatal error.  No further API calls should be made.
 *   * RUI_SDK_ABORTED                               Error:  A required New Registration has failed, and hence the SDK is aborted.  StopSDK and DestroySDK are possible.
 *   * RUI_SDK_PERMANENTLY_DISABLED                  Function validation: The RUI Server has instructed a permanent disable.
 *   * RUI_SDK_OPTED_OUT                             Function validation: The application has instructed an opt out.
 *   * RUI_CONFIG_NOT_CREATED                        Function validation: Configuration has not been successfully created.
 *   * RUI_SDK_ALREADY_STOPPED                       Function validation: SDK has already been successfully stopped.
 *   * RUI_NETWORK_CONNECTION_ERROR                  Network:  Not able to reach the RUI Server.
 *   * RUI_NETWORK_SERVER_ERROR                      Network:  Error while communicating with the RUI Server.
 *   * RUI_NETWORK_RESPONSE_INVALID                  Network:  Message format error while communicating with the RUI Server.
 *   * RUI_TEST_CONNECTION_INVALID_PRODUCT_ID        TestConnection:  Invalid ProductID.
 *   * RUI_TEST_CONNECTION_MISMATCH                  TestConnection:  Mismatch between URL and ProductID.
 */
-(RUIRESULTOBJC)testConnection;

/**
 * Indicates that the RUI SDK should notify the RUI Server that it is opting out.  No further communications with the RUI Server
 * will be made once the opt out is sent.  The client must still call ruiStartSDK to send the opt out message to the RUI Server.
 *
 * optOut() can be called between createConfig:productID:appName:serverURL:protocol:aesKeyHex:multiSessionEnabled:reachOutOnAutoSync:()
 * and startSDK(), and can be called zero or more times.
 *
 * optOut() is a synchronous function, returning when all functionality is completed.
 *
 * @see createConfig:productID:appName:serverURL:protocol:aesKeyHex:multiSessionEnabled:reachOutOnAutoSync:()
 * @see startSDK()
 * @returns   One of the result codes in ruiSDKDefines.h:
 *   * RUI_OK                                        Function (which may be synchronous or asynchronous), fully successful during synchronous functionality.
 *   * RUI_SDK_INTERNAL_ERROR_FATAL                  Error:  Irrecoverable internal fatal error.  No further API calls should be made.
 *   * RUI_SDK_ABORTED                               Error:  A required New Registration has failed, and hence the SDK is aborted.  StopSDK and DestroySDK are possible.
 *   * RUI_SDK_PERMANENTLY_DISABLED                  Function validation: The RUI Server has instructed a permanent disable.
 *   * RUI_SDK_OPTED_OUT                             Function validation: The application has instructed an opt out.
 *   * RUI_CONFIG_NOT_CREATED                        Function validation: Configuration has not been successfully created.
 *   * RUI_SDK_ALREADY_STARTED                       Function validation: SDK has already been successfully started.
 *   * RUI_SDK_ALREADY_STOPPED                       Function validation: SDK has already been successfully stopped.
 */
-(RUIRESULTOBJC)optOut;

/**
 * Explicitly checks for manual ReachOut™ messages on the RUI Server.  checkForReachOutReturningMessage:messageCount:messageType:() 
 * will check for any manual ReachOut™ message type, whereas checkForReachOutOfType:ReturningMessage:messageCount:() will check for
 * ReachOut™ messages of a specified type.
 *
 * checkForReachOutReturningMessage:messageCount:messageType:() can be called between startSDK() and stopSDK:(), and can be
 * called zero or more times.
 *
 * checkForReachOutReturningMessage:messageCount:messageType:() is a synchronous function, returning when all functionality
 * is completed.
 *
 * @see initRegisterDefaultGraphicalReachOutHandler:()
 * @see setReachOutHandler:()
 * @see checkForReachOutOfType:ReturningMessage:messageCount:()
 * @param[out] message          Receives the ReachOut™ string.
 * @param[out] messageCount     Receives the message count (including returned message).
 * @param[out] messageType      Receives the type of returned message.
 * @returns    One of the result codes in RUISDKDefines.h:
 *   * RUI_OK                                        Function (which may be synchronous or asynchronous), fully successful during synchronous functionality.
 *   * RUI_SDK_INTERNAL_ERROR_FATAL                  Error:  Irrecoverable internal fatal error.  No further API calls should be made.
 *   * RUI_SDK_ABORTED                               Error:  A required New Registration has failed, and hence the SDK is aborted.  StopSDK and DestroySDK are possible.
 *   * RUI_SDK_SUSPENDED                             Function validation: The RUI Server has instructed a temporary backoff.
 *   * RUI_SDK_PERMANENTLY_DISABLED                  Function validation: The RUI Server has instructed a permanent disable.
 *   * RUI_SDK_OPTED_OUT                             Function validation: The application has instructed an opt out.
 *   * RUI_CONFIG_NOT_CREATED                        Function validation: Configuration has not been successfully created.
 *   * RUI_SDK_ALREADY_STOPPED                       Function validation: SDK has already been successfully stopped.
 *   * RUI_INVALID_PARAMETER_EXPECTED_NON_NULL       Parameter validation: Some API parameter is expected to be non-NULL, and is not.
 *   * RUI_TIME_THRESHOLD_NOT_REACHED                Function validation: The API call frequency threshold (set by the RUI Server) has not been reached.
 *   * RUI_NETWORK_CONNECTION_ERROR                  Network:  Not able to reach the RUI Server.
 *   * RUI_NETWORK_SERVER_ERROR                      Network:  Error while communicating with the RUI Server.
 *   * RUI_NETWORK_RESPONSE_INVALID                  Network:  Message format error while communicating with the RUI Server.
 */
-(RUIRESULTOBJC)checkForReachOutReturningMessage:(NSMutableString*)message
                                    messageCount:(int32_t*)messageCount
                                     messageType:(int32_t*)messageType;

/**
 * Explicitly checks for manual ReachOut™ messages on the RUI Server.  checkForReachOutOfType:ReturningMessage:messageCount:() 
 * will check for manual ReachOut™ messages of the specified type, whereas checkForReachOutReturningMessage:messageCount:messageType:() 
 * will check for manual ReachOut™ messages of any type.
 *
 * checkForReachOutOfType:ReturningMessage:messageCount:() can be called between startSDK() and stopSDK:(), and can be called
 * zero or more times.
 *
 * checkForReachOutOfType:ReturningMessage:messageCount:() is a synchronous function, returning when all functionality is completed.
 *
 * @see initRegisterDefaultGraphicalReachOutHandler:()
 * @see setReachOutHandler:()
 * @see checkForReachOutReturningMessage:messageCount:messageType:()
 * @param[out] message          Receives the Reachout™ string.
 * @param[out] messageCount     Receives the message count (including returned message).
 * @param[in]  messageType      The message type to search for.
 * @returns    One of the result codes in RUISDKDefines.h:
 *   * RUI_OK                                        Function (which may be synchronous or asynchronous), fully successful during synchronous functionality.
 *   * RUI_SDK_INTERNAL_ERROR_FATAL                  Error:  Irrecoverable internal fatal error.  No further API calls should be made.
 *   * RUI_SDK_ABORTED                               Error:  A required New Registration has failed, and hence the SDK is aborted.  StopSDK and DestroySDK are possible.
 *   * RUI_SDK_SUSPENDED                             Function validation: The RUI Server has instructed a temporary backoff.
 *   * RUI_SDK_PERMANENTLY_DISABLED                  Function validation: The RUI Server has instructed a permanent disable.
 *   * RUI_SDK_OPTED_OUT                             Function validation: The application has instructed an opt out.
 *   * RUI_CONFIG_NOT_CREATED                        Function validation: Configuration has not been successfully created.
 *   * RUI_SDK_ALREADY_STOPPED                       Function validation: SDK has already been successfully stopped.
 *   * RUI_INVALID_MESSAGE_TYPE                      Parameter validation: The messageType is not an allowable value.
 *   * RUI_INVALID_PARAMETER_EXPECTED_NON_NULL       Parameter validation: Some API parameter is expected to be non-NULL, and is not.
 *   * RUI_TIME_THRESHOLD_NOT_REACHED                Function validation: The API call frequency threshold (set by the RUI Server) has not been reached.
 *   * RUI_NETWORK_CONNECTION_ERROR                  Network:  Not able to reach the RUI Server.
 *   * RUI_NETWORK_SERVER_ERROR                      Network:  Error while communicating with the RUI Server.
 *   * RUI_NETWORK_RESPONSE_INVALID                  Network:  Message format error while communicating with the RUI Server.
 */
-(RUIRESULTOBJC)checkForReachOutOfType:(int32_t)messageTypeExpected
                      returningMessage:(NSMutableString*)message
                          messageCount:(int32_t*)messageCount;

/**
 * Checks the RUI Server for the license data for the supplied licenseKey.  Whereas checkLicenseKey:returningLicenseArray:() is
 * a passive check, setLicenseKey:returningLicenseArray:sessionID:() changes the license key.  The license array has size,
 * indexes and values as specified in RUISDKDefines.h:
 *
 * NOTE:  The order of the license array data has changed from the RUI (Trackerbird) SDK V4.
 *
 * checkLicenseKey:returningLicenseArray:() can be called between startSDK() and stopSDK:(), and can be called zero or more times.
 *
 * checkLicenseKey:returningLicenseArray:() is a synchronous function returning when all functionality is completed.
 *
 * @see setLicenseDataKeyType:keyExpired:keyActivated:keyBlocked:keyAllowed:sessionID:()
 * @see setLicenseKey:returningLicenseArray:sessionID:()
 * @param[in]  licenseKey       The license key to check.  Cannot be empty.
 * @param[out] licenseArray     Receives the license data.  NOTE:  Data order changed.
 * @returns    One of the result codes in RUISDKDefines.h:
 *   * RUI_OK                                        Function (which may be synchronous or asynchronous), fully successful during synchronous functionality.
 *   * RUI_SDK_INTERNAL_ERROR_FATAL                  Error:  Irrecoverable internal fatal error.  No further API calls should be made.
 *   * RUI_SDK_ABORTED                               Error:  A required New Registration has failed, and hence the SDK is aborted.  StopSDK and DestroySDK are possible.
 *   * RUI_SDK_SUSPENDED                             Function validation: The RUI Server has instructed a temporary backoff.
 *   * RUI_SDK_PERMANENTLY_DISABLED                  Function validation: The RUI Server has instructed a permanent disable.
 *   * RUI_SDK_OPTED_OUT                             Function validation: The application has instructed an opt out.
 *   * RUI_CONFIG_NOT_CREATED                        Function validation: Configuration has not been successfully created.
 *   * RUI_SDK_ALREADY_STOPPED                       Function validation: SDK has already been successfully stopped.
 *   * RUI_INVALID_PARAMETER_EXPECTED_NON_NULL       Parameter validation: Some API parameter is expected to be non-NULL, and is not.
 *   * RUI_INVALID_PARAMETER_EXPECTED_NON_EMPTY      Parameter validation: Some API parameter is expected to be non-empty, and is not.
 *   * RUI_TIME_THRESHOLD_NOT_REACHED                Function validation: The API call frequency threshold (set by the RUI Server) has not been reached.
 *   * RUI_NETWORK_CONNECTION_ERROR                  Network:  Not able to reach the RUI Server.
 *   * RUI_NETWORK_SERVER_ERROR                      Network:  Error while communicating with the RUI Server.
 *   * RUI_NETWORK_RESPONSE_INVALID                  Network:  Message format error while communicating with the RUI Server.
 */
-(RUIRESULTOBJC) checkLicenseKey:(NSString*)licenseKey
           returningLicenseArray:(NSMutableArray*)licenseArray;

/**
 * Checks the RUI Server for the license data for the supplied licenseKey and sets the current license to licenseKey.
 * Whereas checkLicenseKey:returningLicenseArray:() is a passive check, setLicenseKey:returningLicenseArray:sessionID:() changes
 * the license key.  The RUI Server always registers the licenseKey even if the RUI Server knows nothing about the licenseKey.
 * When a new (unknown) licenseKey is registered, the RUI Server sets the license data to keyType RUI_KEY_TYPE_UNKNOWN and the
 * four status flags (blocked, allowed, expired, activated) to RUI_KEY_STATUS_NO.  The license array has size, indexes
 * and values as specified in RUISDKDefines.h:
 *
 * NOTE:  Different from the V4 of the RUI SDK, a sessionID parameter can be supplied
 * (createConfig:productID:appName:serverURL:protocol:aesKeyHex:multiSessionEnabled:reachOutOnAutoSync:()):
 *   * multiSessionEnabled = false   - sessionID must be empty.  This is similar to event tracking APIs.
 *   * multiSessionEnabled = true    - sessionID must be a current valid value used in startSession:(), or it can be
 *             empty.  This is different than normal event tracking APIs, whereby a empty value is not be allowed.
 *
 * NOTE:  The order of the license array data has changed from the RUI SDK V4.
 *
 * setLicenseKey:returningLicenseArray:sessionID:() can be called between startSDK() and stopSDK:(), and can be called zero or
 * more times.
 *
 * setLicenseKey:returningLicenseArray:sessionID:() is primarily a synchronous function, returning once the check with RUI
 * Server has completed.  Some post-processing functionality is performed asynchronously, executed on separate thread(s).
 *
 * @see setLicenseDataKeyType:keyExpired:keyActivated:keyBlocked:keyAllowed:sessionID:()
 * @see checkLicenseKey:returningLicenseArray:()
 * @param[in]  licenseKey       The license key to check and set as current.  Cannot be empty.
 * @param[out] licenseArray     Receives the license data.  NOTE:  Data order changed.
 * @param[in]  sessionID        A session ID complying with above usage (content conditioning and validation rules in startSession:()).
 * @returns    One of the result codes in RUISDKDefines.h:
 *   * RUI_OK                                        Function (which may be synchronous or asynchronous), fully successful during synchronous functionality.
 *   * RUI_SDK_INTERNAL_ERROR_FATAL                  Error:  Irrecoverable internal fatal error.  No further API calls should be made.
 *   * RUI_SDK_ABORTED                               Error:  A required New Registration has failed, and hence the SDK is aborted.  StopSDK and DestroySDK are possible.
 *   * RUI_SDK_SUSPENDED                             Function validation: The RUI Server has instructed a temporary backoff.
 *   * RUI_SDK_PERMANENTLY_DISABLED                  Function validation: The RUI Server has instructed a permanent disable.
 *   * RUI_SDK_OPTED_OUT                             Function validation: The application has instructed an opt out.
 *   * RUI_CONFIG_NOT_CREATED                        Function validation: Configuration has not been successfully created.
 *   * RUI_SDK_ALREADY_STOPPED                       Function validation: SDK has already been successfully stopped.
 *   * RUI_INVALID_PARAMETER_EXPECTED_NON_NULL       Parameter validation: Some API parameter is expected to be non-NULL, and is not.
 *   * RUI_INVALID_PARAMETER_EXPECTED_NON_EMPTY      Parameter validation: Some API parameter is expected to be non-empty, and is not.
 *   * RUI_INVALID_SESSION_ID_EXPECTED_EMPTY         Parameter validation: The sessionID is expected to be empty, and it was not.
 *   * RUI_INVALID_SESSION_ID_EXPECTED_NON_EMPTY     Parameter validation: The sessionID is expected to be non-empty, and it was not.
 *   * RUI_INVALID_SESSION_ID_TOO_SHORT              Parameter validation: The sessionID violates its allowable min length.
 *   * RUI_INVALID_SESSION_ID_TOO_LONG               Parameter validation: The sessionID violates its allowable max length.
 *   * RUI_INVALID_SESSION_ID_NOT_ACTIVE             Parameter validation: The sessionID is not currently in use.
 *   * RUI_TIME_THRESHOLD_NOT_REACHED                Function validation: The API call frequency threshold (set by the RUI Server) has not been reached.
 *   * RUI_NETWORK_CONNECTION_ERROR                  Network:  Not able to reach the RUI Server.
 *   * RUI_NETWORK_SERVER_ERROR                      Network:  Error while communicating with the RUI Server.
 *   * RUI_NETWORK_RESPONSE_INVALID                  Network:  Message format error while communicating with the RUI Server.
 */
-(RUIRESULTOBJC)setLicenseKey:(NSString*)licenseKey
           returningLicenseArray:(NSMutableArray*)licenseArray
                       sessionID:(NSString*)sessionID;

@end

